@@ -54,6 +54,14 @@ static CmdCfg	*copy_cmc = NULL;
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2011-10-04 -	Return the buffer size for copying, so others can use it too. */
+gsize cpy_get_buf_size(void)
+{
+	return copy_options.buf_size;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 static void cb_progress(goffset pos, goffset total, gpointer user)
 {
 	pgs_progress_item_update(user, pos);
@@ -71,6 +79,40 @@ static gboolean copy_gfile_symlink(MainInfo *min, DirPane *src, DirPane *dst, GF
 	return g_file_copy(from, to, G_FILE_COPY_NOFOLLOW_SYMLINKS, pgs_progress_get_cancellable(), cb_progress, min, err);
 }
 
+static gboolean copy_gfile_special(MainInfo *min, DirPane *src, DirPane *dst, GFile *from, GFile *to, GError **err)
+{
+	GFileInfo	*fi;
+	gchar		*path;
+	gboolean	ok = FALSE;
+
+	if((fi = g_file_query_info(from, "standard::*,unix::*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, err)) == NULL)
+		return FALSE;
+	/* It seems current versions of GIO can't ever set the G_FILE_ATTRIBUTE_UNIX_RDEV attribute
+	 * so we can't just use that. Might be because of the below code required to do this.
+	*/
+	if((path = g_file_get_path(to)) != NULL)
+	{
+		const guint32	mode = g_file_info_get_attribute_uint32(fi, G_FILE_ATTRIBUTE_UNIX_MODE);
+		const guint32	rdev = g_file_info_get_attribute_uint32(fi, G_FILE_ATTRIBUTE_UNIX_RDEV);
+
+		/* Manual page for mknod() says never to use it to create FIFOs, so let's not. */
+		if(mode & S_IFIFO)
+			ok = mkfifo(path, mode) == 0;
+		else
+			ok = mknod(path, mode, rdev) == 0;
+
+		if(!ok)
+		{
+			const gchar	*fmt = (mode & S_IFIFO) ? _("Error copying FIFO: %s") : _("Error copying special file: %s");
+			g_set_error(err, g_io_error_quark(), g_io_error_from_errno(errno), fmt, strerror(errno));
+		}
+		g_free(path);
+	}
+	g_object_unref(fi);
+
+	return ok;
+}
+
 static gboolean copy_gfile_directory(MainInfo *min, DirPane *src, DirPane *dst, GFile *from, GFile *to, GError **err)
 {
 	GFileEnumerator	*fe;
@@ -148,6 +190,11 @@ gboolean copy_gfile(MainInfo *min, DirPane *src, DirPane *dst, GFile *from, GFil
 		ok = copy_gfile_symlink(min, src, dst, from, to, err);
 		pgs_progress_item_end(min);
 		break;
+	case G_FILE_TYPE_SPECIAL:
+		pgs_progress_item_begin(min, g_file_info_get_display_name(fi), 0);
+		ok = copy_gfile_special(min, src, dst, from, to, err);
+		pgs_progress_item_end(min);
+		break;
 	default:
 		g_error("Not copying '%s', type is %d which is unsupported", g_file_info_get_name(fi), g_file_info_get_file_type(fi));
 		ok = FALSE;
@@ -217,7 +264,7 @@ void cfg_copy(MainInfo *min)
 		cmc_field_add_boolean(copy_cmc, "copy_dates", _("Preserve Dates During Copy?"), offsetof(OptCopy, copy_dates));
 		cmc_field_add_boolean(copy_cmc, "ignore_attrib_err", _("Ignore Failure to Copy Attributes (Date, Owner, Mode)?"), offsetof(OptCopy, ignore_attrib_err));
 		cmc_field_add_boolean(copy_cmc, "leave_fullsize", _("Leave Failed Destination if Full Size?"), offsetof(OptCopy, leave_fullsize));
-		cmc_field_add_size(copy_cmc, "buf_size", _("Buffer Size"), offsetof(OptCopy, buf_size), 1024, (1<<20), 1024);
+		cmc_field_add_size(copy_cmc, "buf_size", _("Buffer Size"), offsetof(OptCopy, buf_size), 1024, (1<<24), 1024);
 		cmc_config_register(copy_cmc);
 	}
 }
