@@ -205,7 +205,7 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
 		icon = gtk_image_new_from_stock(dp_row_get_flags(m, row, DPRF_LINK_EXISTS) ? GTK_STOCK_YES : GTK_STOCK_NO, GTK_ICON_SIZE_BUTTON);
 		gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 0);
-		gtk_table_attach(GTK_TABLE(tab), hbox, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL,5,0);
+		gtk_table_attach(GTK_TABLE(tab), hbox, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
 		y++;
 	}
 	sep = gtk_hseparator_new();
@@ -301,7 +301,7 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2003-10-16 -	Callback when Information window is closing. Stop asynch. dir sizing, if in use. */
-static void info_close(gint button, gpointer user)
+static void info_close(GtkWidget *wid, GdkEvent *event, gpointer user)
 {
 	DirSizeInfo	*dsi = user;
 
@@ -318,8 +318,13 @@ static gint information(MainInfo *min, const DirPane *dp, const DirRow2 *row)
 
 	if((fr = build_info(min, dp, row, &dsi)) != NULL)
 	{
-		gtk_widget_show_all(fr);
-		dlg_dialog_async_new(fr, dp_row_get_name_display(dp_get_tree_model(dp), row), _("_OK"), info_close, dsi);
+		GtkWidget	*win;
+
+		win = win_window_open(min->cfg.wininfo, WIN_INFO);
+		win_window_set_title(win, dp_row_get_name_display(dp_get_tree_model(dp), row));
+		g_signal_connect(G_OBJECT(win), "delete_event", G_CALLBACK(info_close), dsi);
+		gtk_container_add(GTK_CONTAINER(win), fr);
+		win_window_show(win);
 	}
 	return fr != NULL;
 }
