@@ -126,7 +126,6 @@ Dialog * dlg_dialog_sync_new(GtkWidget *body, const gchar *title, const gchar *b
 	gtk_window_set_position(GTK_WINDOW(dlg->dlg), the_state.window_pos);
 	g_signal_connect(GTK_OBJECT(dlg->dlg), "key_press_event", G_CALLBACK(evt_sync_key_pressed), dlg);
 	g_signal_connect(GTK_OBJECT(dlg->dlg), "response", G_CALLBACK(evt_sync_response), dlg);
-	gtk_dialog_set_default_response(GTK_DIALOG(dlg->dlg), 0);
 	gtk_window_set_transient_for(GTK_WINDOW(dlg->dlg), the_state.main_window);
 	dlg->stay_open = FALSE;
 	if(title != NULL)
@@ -143,6 +142,7 @@ Dialog * dlg_dialog_sync_new(GtkWidget *body, const gchar *title, const gchar *b
 		gtk_widget_show_all(vbox);
 	}
 	dlg->last_button = add_buttons(dlg->dlg, buttons);
+	gtk_dialog_set_default_response(GTK_DIALOG(dlg->dlg), 0);
 
 	return dlg;
 }
@@ -302,6 +302,12 @@ void dlg_dialog_async_close_silent(Dialog *dlg)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2011-09-28 -	Return the GtkDialog, which can be useful sometimes. */
+GtkDialog * dlg_dialog_get_dialog(const Dialog *dlg)
+{
+	return dlg != NULL ? GTK_DIALOG(dlg->dlg) : NULL;
+}
+
 static void evt_entry_changed(GtkWidget *entry, gpointer user)
 {
 	const gchar	*text = gtk_entry_get_text(GTK_ENTRY(entry));
