@@ -275,14 +275,14 @@ void dpf_init_defaults(CfgInfo *cfg)
 	fmt->sort.invert  = FALSE;
 	fmt->sort.mode    = DPS_DIRS_FIRST;
 
-	strcpy(fmt->def_path, ".");
+	strcpy(fmt->def_path, "file:///");
 	fmt->path_above = FALSE;
 	fmt->hide_allowed = TRUE;
 	fmt->scrollbar_always = FALSE;
 
 	cfg->dp_format[1] = cfg->dp_format[0];
 
-	strcpy(cfg->dp_format[1].def_path, ".");
+	strcpy(cfg->dp_format[1].def_path, "file:///");
 
 	cfg->dp_paning.orientation = DPORIENT_HORIZ;
 	cfg->dp_paning.mode	   = DPSPLIT_RATIO;
