@@ -29,11 +29,6 @@
 
 #define	MIN_COPY_CHUNK	(1 << 17)
 
-/* Path components are relative to this if they're not absolute. */
-static gchar	base_path[PATH_MAX];
-/* The length of the base. Saves some calls to strlen() in the middle of everything. :) */
-static gint	base_len;
-
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-05-21 -	Enter a directory, saving the old path for restoration later.
@@ -212,336 +207,249 @@ gboolean fut_is_parent_of(const char *from, const char *to)
 */
 gboolean fut_exists(const gchar *name)
 {
-	gint		old_errno = errno;
-	gboolean	ret;
-
-	ret = (access(name, F_OK) == 0);
-	errno = old_errno;
-
-	return ret;
-}
-
-/* 1998-12-18 -	Sets <size> to the size of file <name>, in bytes. Returns 1 on success,
-**		0 on failure (in case <size> is never touched).
-*/
-gboolean fut_size(const gchar *name, gsize *size)
-{
-	struct stat	s;
-	gint		old_errno = errno, ret = 0;
-
-	if(name != NULL && size != NULL && (ret = (stat(name, &s)) == 0))
-		*size = (gsize) s.st_size;
-	errno = old_errno;
-	return ret ? TRUE : FALSE;
-}
-
-/* 1999-02-03 -	Works just like fut_size above, but takes an open file descriptor
-**		instead of a filename. Sometimes handy.
-*/
-gboolean fut_fsize(gint fd, gsize *size)
-{
-	struct stat	s;
-	gint		old_errno = errno, ret = 0;
-
-	if(fd >= 0 && size != NULL && (ret = (fstat(fd, &s)) == 0))
-		*size = (gsize) s.st_size;
-	errno = old_errno;
-	return ret ? TRUE : FALSE;
+	return access(name, F_OK) == 0;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-12-20 -	This is under the hood of the fut_dir_size() function. */
-static gint recurse_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc, gboolean with_progress)
+static gboolean	do_size_gfile_info(MainInfo *min, const GFile *object, const GFileInfo *fi, guint64 *bytes, FUCount *fc, GError **error);
+
+static gboolean do_size_gfile_dir(MainInfo *min, const GFile *root, guint64 *bytes, FUCount *fc, GError **error)
 {
-	gchar		old_dir[PATH_MAX];
-	DIR		*dir;
-	struct dirent	*de;
-	struct stat	stat;
+	GFileEnumerator	*fe;
+	GFileInfo	*fi;
+	GFile		*child;
+	gboolean	ok = TRUE;
 
-	if(path == NULL)
-		return 0;
-
-	if(fut_cd(path, old_dir, sizeof old_dir))
+	if((fe = g_file_enumerate_children((GFile *) root, "standard::*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, error)) == NULL)
+		return FALSE;
+	while(ok && ((fi = g_file_enumerator_next_file(fe, NULL, error)) != NULL))
 	{
-		if((dir = opendir(".")) != NULL)
+		if((child = g_file_get_child_for_display_name((GFile *) root, g_file_info_get_display_name(fi), NULL)) != NULL)
 		{
-			err_clear(min);
-			while((de = readdir(dir)) != NULL)
-			{
-				if(with_progress)
-				{
-					pgs_progress_item_begin(min, de->d_name, 0);
-					if(pgs_progress_item_update(min, 0) == PGS_CANCEL)
-					{
-						errno = EINTR;
-						break;
-					}
-				}
-				if(!min->cfg.dir_filter(de->d_name))
-					continue;
-				if(lstat(de->d_name, &stat) == 0)
-				{
-					if(bytes != NULL)
-						(*bytes) += stat.st_size;
-					if(fc != NULL)
-					{
-						fc->num_bytes  += (guint64) stat.st_size;
-						fc->num_blocks += (guint64) stat.st_blocks;
-					}
-					if(S_ISDIR(stat.st_mode))
-					{
-						if(fc != NULL)
-							fc->num_dirs++;
-						recurse_dir_size(min, de->d_name, bytes, fc, with_progress);
-					}
-					else if(!S_ISDIR(stat.st_mode) && fc != NULL)
-					{
-						if(S_ISREG(stat.st_mode))
-							fc->num_files++;
-						else if(S_ISLNK(stat.st_mode))
-							fc->num_links++;
-						else if(S_ISCHR(stat.st_mode))
-							fc->num_cdevs++;
-						else if(S_ISBLK(stat.st_mode))
-							fc->num_bdevs++;
-						else if(S_ISFIFO(stat.st_mode))
-							fc->num_fifos++;
-						else if(S_ISSOCK(stat.st_mode))
-							fc->num_socks++;
-					}
-				}
-				if(with_progress)
-					pgs_progress_item_end(min);
-			}
-			closedir(dir);
+			ok = do_size_gfile_info(min, child, fi, bytes, fc, error);
+			g_object_unref(child);
 		}
-		fut_cd(old_dir, NULL, 0);
+		else
+			ok = FALSE;
+		g_object_unref(fi);
 	}
-	return errno == 0;
+	if(ok && fc != NULL)
+		fc->num_total = fc->num_dirs + fc->num_files + fc->num_links + fc->num_specials;
+
+	g_object_unref(fe);
+
+	return ok;
 }
 
-static gint do_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc, gboolean with_progress)
+static gboolean do_size_gfile_info(MainInfo *min, const GFile *object, const GFileInfo *fi, guint64 *bytes, FUCount *fc, GError **error)
 {
-	gint	ret;
+	gboolean	ret = TRUE;
 
+	if(bytes != NULL)
+		*bytes += g_file_info_get_size((GFileInfo *) fi);
+	if(fc != NULL)
+	{
+		fc->num_bytes += g_file_info_get_size((GFileInfo *) fi);
+		fc->num_blocks += g_file_info_get_attribute_uint64((GFileInfo *) fi, G_FILE_ATTRIBUTE_UNIX_BLOCKS);
+	}
+	switch(g_file_info_get_file_type((GFileInfo *) fi))
+	{
+	case G_FILE_TYPE_REGULAR:
+		if(fc != NULL)
+			fc->num_files++;
+		break;
+	case G_FILE_TYPE_DIRECTORY:
+		if(fc != NULL)
+			fc->num_dirs++;
+		ret = do_size_gfile_dir(min, object, bytes, fc, error);
+		break;
+	case G_FILE_TYPE_SYMBOLIC_LINK:
+		if(fc != NULL)
+			fc->num_links++;
+		break;
+	case G_FILE_TYPE_SPECIAL:
+		/* FIXME: This is not supported in the GIO world ... Has to go. */
+		break;
+	default:
+		g_warning("Can't handle irregular file type in do_size_gfile_info()");
+	}
+	return ret;
+}
+
+gboolean fut_size_gfile(MainInfo *min, const GFile *file, guint64 *bytes, FUCount *fc, GError **error)
+{
+	GFileInfo	*fi;
+	gboolean	ret = TRUE;
+
+	if(min == NULL || file == NULL)
+		return FALSE;
+	if((fi = g_file_query_info((GFile *) file, "standard::*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, error)) == NULL)
+		return FALSE;
 	if(fc != NULL)		/* If there is a fc present, wipe it clean. */
 	{
 		fc->num_dirs  = 0U;
 		fc->num_files = 0U;
 		fc->num_links = 0U;
-		fc->num_cdevs = 0U;
-		fc->num_bdevs = 0U;
-		fc->num_fifos = 0U;
-		fc->num_socks = 0U;
+		fc->num_specials = 0U;
 
 		fc->num_total = 0U;
 
 		fc->num_bytes  = 0ULL;
 		fc->num_blocks = 0ULL;
 	}
-	if((ret = recurse_dir_size(min, path, bytes, fc, with_progress)) != 0 && fc != NULL)
-		fc->num_total = fc->num_dirs + fc->num_files + fc->num_links + fc->num_cdevs +
-				fc->num_bdevs + fc->num_fifos + fc->num_socks;
+	ret = do_size_gfile_info(min, file, fi, bytes, fc, error);
+	/* Adjust, don't consider a directory to contain itself. */
+	if(g_file_info_get_file_type(fi) == G_FILE_TYPE_DIRECTORY && fc != NULL)
+		fc->num_dirs--;
+  	g_object_unref(fi);
 
 	return ret;
 }
 
-/* 1998-12-19 -	Scan recursively at <path>, accumulating total number of bytes seen, as well as counting
-**		how many different "objects" (dirs, files, links etc) were seen. Will never follow a link
-**		to a dir or count it as anything but a soft link. <fc> can be NULL if you just don't care.
-**		Returns 1 on success, 0 on failure (such as permission denied).
-** 1999-03-22 -	Now zeros the <bytes> value before beginning. This fixes a bug in the GetSize command.
-*/
-gint fut_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc)
-{
-	return do_dir_size(min, path, bytes, fc, FALSE);
-}
-
-/* 1999-03-28 -	Size up a directory, with GUI progress indication. */
-gint fut_dir_size_progress(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc)
-{
-	return do_dir_size(min, path, bytes, fc, TRUE);
-}
-
 /* ----------------------------------------------------------------------------------------- */
 
-/* Transversal state for a single directory when doing asynchronous dir sizing. */
 typedef struct {
-	DIR 	*dir;
-	gchar	*path;
-} DirSizeState;
+	GStaticMutex	mutex;	/* Lock this before accessing counts. */
+	gboolean	changed;	/* This is set if the data has changed. */
+	FUCount		counts;
+} FUCountSafe;
 
-/* An "operation" is the result of a single call to fut_dir_size_start(). */
 typedef struct {
+	GFile		*root;
+	GCancellable	*cancel;
+	FUCountSafe	counts;		/* Here's where the thread updates. */
+	guint		depth;
 	SizeFunc	func;
-	gpointer	user;
-	FUCount	count;
-	DirSizeState	*current;
-	GSList	*stack;
-} DirSizeOp;
+	gpointer	func_user;
+	guint		timeout;
+	GThread		*thread;
+} SizeGFileInfo;
 
-/* Global state for asynchronous directory size computation. */
-static struct {
-	guint	idle;
-	GList	*ops;
-} dir_size_info = { 0U };
-
-/* 2003-10-16 -	Open given path and return new dir size state struct describing it. */
-static DirSizeState * dir_size_open(const gchar *path)
+static void thread_recurse_directory(GFile *root, SizeGFileInfo *info)
 {
-	DirSizeState	*st;
+	GFileEnumerator	*fen;
 
-	st = g_malloc(sizeof *st);
-	if((st->dir = opendir(path)) != NULL)
+	if((fen = g_file_enumerate_children(root, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, info->cancel, NULL)) != NULL)
 	{
-		st->path = g_strdup(path);
-		return st;
+		GFileInfo	*fin;
+
+		while(!g_cancellable_is_cancelled(info->cancel) && (fin = g_file_enumerator_next_file(fen, info->cancel, NULL)) != NULL)
+		{
+			const GFileType	type = g_file_info_get_file_type(fin);
+
+			/* Grab the lock, and increase the proper counter based on type. */
+			g_static_mutex_lock(&info->counts.mutex);
+			info->counts.counts.num_total++;
+			if(type == G_FILE_TYPE_REGULAR)
+			{
+				info->counts.counts.num_files++;
+				info->counts.counts.num_bytes += g_file_info_get_size(fin);
+			}
+			else if(type == G_FILE_TYPE_DIRECTORY)
+				info->counts.counts.num_dirs++;
+			else if(type == G_FILE_TYPE_SYMBOLIC_LINK)
+				info->counts.counts.num_links++;
+			else if(type == G_FILE_TYPE_SPECIAL)
+				info->counts.counts.num_specials++;
+			info->counts.changed = TRUE;
+			/* All done, release the lock for the rest. */
+			g_static_mutex_unlock(&info->counts.mutex);
+
+			/* Wait now, if that was a directory we need to enter it! */
+			if(type == G_FILE_TYPE_DIRECTORY)
+			{
+				GFile	*child;
+
+				if((child = g_file_get_child(root, g_file_info_get_name(fin))) != NULL)
+				{
+					info->depth++;
+					thread_recurse_directory(child, info);
+					info->depth--;
+					g_object_unref(child);
+				}
+			}
+		}
+		g_file_enumerator_close(fen, info->cancel, NULL);
+		g_object_unref(G_OBJECT(fen));
 	}
-	g_free(st);
+}
+
+static gpointer size_gfile_thread_func(gpointer data)
+{
+	SizeGFileInfo	*info = data;
+
+	thread_recurse_directory(info->root, info);
+
 	return NULL;
 }
 
-/* 2003-10-16 -	Free a directory state. */
-static void dir_size_free(DirSizeState *state)
+static gboolean cb_size_gfile_timeout(gpointer user)
 {
-	if(state->dir)
-		closedir(state->dir);
-	g_free(state->path);
-	g_free(state);
-}
+	SizeGFileInfo	*info = user;
 
-/* 2003-10-16 -	Update asynchronous directory sizing. */
-static gboolean dir_size_update(DirSizeOp *op)
-{
-	int	i;
-	struct dirent	*de;
-	struct stat	stbuf;
-
-	for(i = 0; op->current && i < 32; i++)
+	/* Has the data changed? */
+	if(info->counts.changed)
 	{
-		if((de = readdir(op->current->dir)) != NULL)
-		{
-			gchar	full[PATH_MAX];
+		FUCount	local;
 
-			if(strcmp(de->d_name, ".") == 0 || strcmp(de->d_name, "..") == 0)
-				continue;
-			/* Don't CD for asynchronous reasons. Build full paths, it's fairly cheap. */
-			g_snprintf(full, sizeof full, "%s%s%s", op->current->path, G_DIR_SEPARATOR_S, de->d_name);
-			if(lstat(full, &stbuf) == 0)
-			{
-				op->count.num_bytes  += (guint64) stbuf.st_size;
-				op->count.num_blocks += (guint64) stbuf.st_blocks;
-				if(S_ISDIR(stbuf.st_mode))
-				{
-					DirSizeState	*nd;
+		/* Try to quickly create a local copy. */
+		g_static_mutex_lock(&info->counts.mutex);
+		local = info->counts.counts;
+		info->counts.changed = FALSE;
+		g_static_mutex_unlock(&info->counts.mutex);
 
-					if((nd = dir_size_open(full)) != NULL)
-					{
-						op->stack   = g_slist_prepend(op->stack, op->current);
-						op->current = nd;
-						op->count.num_dirs++;
-					}
-				}
-				else
-				{
-					if(S_ISREG(stbuf.st_mode))
-						op->count.num_files++;
-					else if(S_ISLNK(stbuf.st_mode))
-						op->count.num_links++;
-					else if(S_ISCHR(stbuf.st_mode))
-						op->count.num_cdevs++;
-					else if(S_ISBLK(stbuf.st_mode))
-						op->count.num_bdevs++;
-					else if(S_ISFIFO(stbuf.st_mode))
-						op->count.num_fifos++;
-					else if(S_ISSOCK(stbuf.st_mode))
-						op->count.num_socks++;
-				}
-			}
-		}
-
-		if(de == NULL)
-		{
-			dir_size_free(op->current);
-			op->current = NULL;
-			if(op->stack != NULL)
-			{
-				GSList	*top = op->stack;
-
-				op->stack = g_slist_remove_link(op->stack, top);
-				op->current = top->data;
-				g_slist_free_1(top);
-			}
-		}
+		/* Then call callback using the local copy, leaving the true data free to
+		 * update by the thread while the GUI-building code in the callback runs.
+		 */
+		info->func(&local, info->func_user);
 	}
-	/* After looping around for a while, tell op owner new numbers. */
-	op->func(&op->count, op->user);
-	return op->current != NULL;
+	return !g_cancellable_is_cancelled(info->cancel);
 }
 
-/* 2003-10-16 -	GTK+ idle callback for asynchronous directory sizing. Update all on-going operations. */
-static gint idle_dir_size(gpointer user)
+gpointer fut_dir_size_gfile_start(GFile *dir, SizeFunc func, gpointer user)
 {
-	GList	*iter, *next;
+	SizeGFileInfo	*info;
 
-	for(iter = dir_size_info.ops; iter != NULL; iter = next)
-	{
-		next = g_list_next(iter);
-		if(!dir_size_update(iter->data))
-			fut_dir_size_stop(iter->data);
-	}
-	return TRUE;
+	if(dir == NULL || func == NULL)
+		return NULL;
+
+	info = g_malloc(sizeof *info);
+	info->root = dir;
+	info->cancel = g_cancellable_new();
+	g_static_mutex_init(&info->counts.mutex);
+	info->counts.changed = FALSE;
+	memset(&info->counts.counts, 0, sizeof info->counts.counts);
+	info->depth = 0;
+	info->func = func;
+	info->func_user = user;
+
+	info->timeout = g_timeout_add(350, cb_size_gfile_timeout, info);
+
+	info->thread = g_thread_create(size_gfile_thread_func, info, TRUE, NULL);
+
+	return info;
 }
 
-/* 2003-10-16 -	Start off a new operation, by allocating, initializing it, and adding it to list. */
-gpointer fut_dir_size_start(const gchar *path, SizeFunc func, gpointer user)
+void fut_dir_size_gfile_stop(gpointer handle)
 {
-	DirSizeOp	*op;
+	SizeGFileInfo	*info = handle;
 
-	op = g_malloc(sizeof *op);
-	op->func = func;
-	op->user = user;
-	memset(&op->count, 0, sizeof op->count);
-	op->current = dir_size_open(path);
-	op->stack = NULL;
+	if(info == NULL)
+		return;
+	/* If we remove this first, and we know we're not in it now (main thread only), we can be
+	 * pretty sure it's safe to tear down the rest since the timeout function won't run again.
+	 */
+	g_source_remove(info->timeout);
 
-	dir_size_info.ops = g_list_append(dir_size_info.ops, op);
+	/* Now, stop the thread and kill everything. */
+	g_cancellable_cancel(info->cancel);
+	g_thread_join(info->thread);
+	if(info->counts.changed)
+		info->func(&info->counts.counts, info->func_user);
 
-	/* If the idle function isn't active, start it up. */
-	if(dir_size_info.idle == 0U)
-		dir_size_info.idle = gtk_idle_add_priority(G_PRIORITY_LOW, idle_dir_size, NULL);
-	return op;
-}
-
-/* 2003-10-16 -	Stop a directory sizing operation. */
-void fut_dir_size_stop(gpointer handle)
-{
-	DirSizeOp	*op = handle;
-	GList	*link;
-
-	if((link = g_list_find(dir_size_info.ops, op)) != NULL)
-	{
-		GSList	*iter;
-
-		dir_size_info.ops = g_list_remove(dir_size_info.ops, op);
-
-		op->func(NULL, op->user);
-
-		if(op->current)
-			dir_size_free(op->current);
-		for(iter = op->stack; iter != NULL; iter = g_slist_next(iter))
-			dir_size_free(iter->data);
-		g_slist_free(op->stack);
-		g_free(op);
-
-		/* If no more operations active, stop the idle function too. */
-		if(dir_size_info.ops == NULL)
-		{
-			gtk_idle_remove(dir_size_info.idle);
-			dir_size_info.idle = 0U;
-		}
-	}
+	g_object_unref(G_OBJECT(info->cancel));
+	g_free(info);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -646,7 +554,7 @@ static void eval_env(gchar *component)
 	}
 	if(temp)
 	{
-		stu_strncpy(component, temp->str, PATH_MAX);
+		g_strlcpy(component, temp->str, PATH_MAX);
 		g_string_free(temp, TRUE);
 	}
 }
@@ -671,7 +579,7 @@ static void eval_home(gchar *component)
 			g_string_assign(temp, "");
 		g_string_append(temp, ptr);
 		if(*temp->str)
-			stu_strncpy(component, temp->str, PATH_MAX);
+			g_strlcpy(component, temp->str, PATH_MAX);
 		else
 			g_snprintf(component, PATH_MAX, "~%s", uname);
 		g_string_free(temp, TRUE);
@@ -684,6 +592,7 @@ void fut_interpolate(gchar *buffer)
 	eval_env(buffer);
 	if(buffer[0] == '~')
 		eval_home(buffer);
+#if 0	/* FIXME: This needs to become URL-aware, in a big way. */
 	else if(buffer[0] != G_DIR_SEPARATOR && base_path[0] == G_DIR_SEPARATOR)
 	{
 		gint	len = strlen(buffer);
@@ -696,6 +605,7 @@ void fut_interpolate(gchar *buffer)
 			buffer[base_len] = G_DIR_SEPARATOR;
 		}
 	}
+#endif
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -816,81 +726,6 @@ void fut_free_path(GList *list)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-11-13 -	Basic low-level file copying routine. Reads from <in>, writes into <out>,
-**		both of which must be open with the appropriate modes. Copies by reading
-**		<chunk_size> bytes at a time. Will handle files having 0 as their size, like
-**		most of the fun things in the /proc directory (on Linux systems).
-**		Returns the number of bytes successfully written to destination. Note that
-**		for empty files, a return value of 0 indicates success.
-** 2000-07-16 -	Bug fix: the interruption check was done after the write(), so small files
-**		that fit into the chunk_size could not be interrupted. Oops.
-** 2006-01-06 -	When writing across a SMB mount, write() seems to dislike large chunks, so
-**		we need to do the write()s multiple times for each read(). Odd.
-*/
-off_t fut_copy(gint in, gint out, gsize chunk_size)
-{
-	gpointer	buf;
-	gconstpointer	wbuf;
-	gssize		got, wrote, to_write;
-	off_t		put = 0U;
-
-	if((in < 0) || (out < 0))
-		return 0;
-
-	chunk_size = MAX(chunk_size, MIN_COPY_CHUNK);
-
-	buf = g_malloc(chunk_size);
-	while((got = read(in, buf, chunk_size)) > 0)
-	{
-		if(!pgs_progress_item_update(NULL, put))
-		{
-			errno = EINTR;
-			break;
-		}
-		for(to_write = got, wbuf = buf; to_write > 0; to_write -= wrote, wbuf += wrote, put += wrote)
-		{
-			wrote = write(out, wbuf, to_write);
-			if(wrote < 0)
-			{
-				fprintf(stderr, "gentoo: fut_copy() failed to write %" G_GSSIZE_FORMAT "-byte block, errno=%d--aborting copy\n", got, errno);
-				goto fail;
-			}
-		}
-	}
-fail:
-	g_free(buf);
-
-	return put;
-}
-
-/* 1999-11-13 -	Copy just <length> bytes from <in> to <out>. */
-off_t fut_copy_partial(gint in, gint out, gsize chunk_size, gsize length)
-{
-	gpointer	buf;
-	gssize		got, wrote, block;
-	off_t		put = 0U;
-
-	chunk_size = MAX(MIN_COPY_CHUNK, MIN(chunk_size, length));
-
-	buf = g_malloc(chunk_size);
-	while(put < length)
-	{
-		block = (put + chunk_size <= length) ? chunk_size : length - put;
-		got = read(in, buf, block);
-		if(got <= 0)
-			break;
-		wrote = write(out, buf, block);
-		if(wrote < 0)
-			break;
-		put += block;
-	}
-	g_free(buf);
-
-	return put;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-09-07 -	Check if the user described by (<uid>,<gid>), which is the user currently running
 **		gentoo, is allowed to READ from a file described by <stat>. Note that this is a
 **		complex check.
@@ -950,33 +785,6 @@ gboolean fut_can_execute(const struct stat *stat, uid_t uid, gid_t gid)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-05-29 -	Check if there is a file named <name> in <path>. If such a file exists, check
-**		if the current user can execute it. This is handy as a prelude to actually doing
-**		a fork()/exec() pair on <name>, since it allows simpler error handling. If execution
-**		is not possible, set errno to something suitable and return FALSE. If everything
-**		looks fins (not a guarantee exec() will succeed), clear errno and return TRUE.
-*/
-gboolean fut_locate_executable(const gchar *path, const gchar *name)
-{
-	errno = EINVAL;		/* Assume the worst. */
-
-	if((path != NULL) && (name != NULL))
-	{
-		const gchar	*pname;
-
-		if((pname = fut_locate(path, name)) != NULL)
-		{
-			if(access(pname, X_OK) == 0)
-				return TRUE;
-		}
-		else
-			errno = ENOENT;
-	}
-	return FALSE;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-10-26 -	Check if the user doesn't want to see <name>. Returns TRUE if the file is
 **		NOT hidden, FALSE if it is. If <name> is NULL, we free any allocated resources
 **		and then return TRUE. This is very useful when the RE source is changing.
@@ -1027,24 +835,3 @@ gboolean fut_check_hide(DirPane *dp, const gchar *name)
 	}
 	return TRUE;					/* When in doubt, hide nothing. */
 }
-
-/* ----------------------------------------------------------------------------------------- */
-
-/* 1998-08-30 -	Initialize the file utility module. Grabs the current working directory for
-**		use as the global 'base' for paths. Run this early!
-*/
-void fut_initialize(void)
-{
-	if(getcwd(base_path, sizeof base_path) == NULL)
-	{
-		fprintf(stderr, "FILEUTIL: Couldn't get current working dir - defaulting to /\n");
-		stu_strncpy(base_path, "/", sizeof base_path);
-	}
-	base_len = strlen(base_path);
-}
-
-/* 2003-11-10 -	Change back to the directory saved in fut_initialize(). Might help profiling. */
-void fut_finalize(void)
-{
-	fut_cd(base_path, NULL, 0U);
-}
