@@ -11,6 +11,10 @@
 #include <time.h>
 #include <sys/wait.h>
 
+#if defined ENABLE_NLS
+#include <locale.h>
+#endif
+
 #include "errors.h"
 #include "configure.h"
 #include "dirpane.h"
@@ -281,8 +285,16 @@ static GtkWidget * build_gui(MainInfo *min)
 /* 1998-08-23 -	Initialize the paths config data. */
 static void init_paths(CfgInfo *cfg)
 {
+	const gchar	*confdir;
+
 	cfg->path.path[PTID_ICON]  = g_string_new(PATH_ICN);
-	cfg->path.path[PTID_GTKRC] = g_string_new(PATH_GRC);
+	if((confdir = g_get_user_config_dir()) != NULL)
+	{
+		cfg->path.path[PTID_GTKRC] = g_string_new(confdir);
+		g_string_append(cfg->path.path[PTID_GTKRC], G_DIR_SEPARATOR_S PACKAGE);
+	}
+	else
+		cfg->path.path[PTID_GTKRC] = g_string_new(PATH_GRC);
 	cfg->path.path[PTID_FSTAB] = g_string_new("/etc/fstab");
 	cfg->path.path[PTID_MTAB]  = g_string_new("/proc/mounts");
 
@@ -297,15 +309,22 @@ static void init_paths(CfgInfo *cfg)
 **		file is always named ".gentoogtkrc" (too long, I know) but you can put
 **		it anywhere as long as you given gentoo the path (in the config).
 ** 2001-08-12 -	The dot is now optional, but is given priority if found.
+** 2011-07-24 -	Now we even prefer "gtkrc", without "gentoo", to match .config/gentoo location.
 */
 static void load_gtk_rc(MainInfo *min)
 {
+	const gchar	*names[] = { "gtkrc", "gentoogtkrc", ".gentoogtkrc" };
 	const gchar	*name;
+	gsize		i;
 
-	if((name = fut_locate(min->cfg.path.path[PTID_GTKRC]->str, ".gentoogtkrc")) != NULL)
-		gtk_rc_parse(name);
-	else if((name = fut_locate(min->cfg.path.path[PTID_GTKRC]->str, "gentoogtkrc")) != NULL)
-		gtk_rc_parse(name);
+	for(i = 0; i < sizeof names / sizeof *names; i++)
+	{
+		if((name = fut_locate(min->cfg.path.path[PTID_GTKRC]->str, names[i])) != NULL)
+		{
+			gtk_rc_parse(name);
+			break;
+		}
+	}
 }
 
 /* 1999-04-04 -	Initialize default (er, and non-configurable) command options. I'll
@@ -410,7 +429,7 @@ int main(int argc, char *argv[])
 	gboolean	show_version = FALSE;
 	gboolean	show_locale_info = FALSE;
 	gboolean	root_ok = FALSE;
-	gboolean	no_rc = FALSE, no_gtk_rc = FALSE;
+	gboolean	no_rc = FALSE, no_gtk_rc = FALSE, no_dir_history = FALSE;
 	gchar		*dir_left = NULL;
 	gchar		*dir_right = NULL;
 	GOptionEntry	option_entries[] = {
@@ -421,6 +440,7 @@ int main(int argc, char *argv[])
 	{ "root-ok", 0, 0, G_OPTION_ARG_NONE, &root_ok, N_("Allows gentoo to be run by the root user. Could be dangerous"), NULL },
 	{ "no-rc", 0, 0, G_OPTION_ARG_NONE, &no_rc, N_("Do not load the ~/.gentoorc configuration file; instead, use default values"), NULL },
 	{ "no-gtk-rc", 0, 0, G_OPTION_ARG_NONE, &no_gtk_rc, N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults"), NULL },
+	{ "no-dir-history", 0, 0, G_OPTION_ARG_NONE, &no_dir_history, N_("Do not load the dirhistory file"), NULL },
 	{ "run", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &min.run_commands, N_("Run COMMAND, a gentoo command. Done before user interaction allowed, but after configuration "
 			      "file has been read in. Can be used many times to run several commands in sequence"), N_("COMMAND") },
 	{ "left", '1', 0, G_OPTION_ARG_STRING, &dir_left, N_("Use DIR as path for the left directory pane. Overrides default (and history)"), N_("DIR") },
@@ -542,7 +562,8 @@ int main(int argc, char *argv[])
 
 		gtk_container_add(GTK_CONTAINER(min.gui->window), box);
 
-		dph_history_load(&min, min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
+		if(!no_dir_history)
+			dph_history_load(&min, min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
 
 		gtk_window_set_resizable(GTK_WINDOW(min.gui->window), TRUE);
 
