@@ -81,7 +81,7 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
-#define	RCNAME	".gentoorc"
+#define	RCNAME	"gentoorc"
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -91,14 +91,8 @@
 #include "gnu-gettext.h"
 
 #if defined ENABLE_NLS
-#include <locale.h>
-/* These two are used to mark translatable strings in the source code.
-** _() is used in contexts where C allows a function call, N_() where
-** it does not. Strings marked with N_() still need to be passed to _()
-** before being shown to the user.
-*/
-#define	_(str)	gettext(str)
-#define	N_(str)	(str)
+/* This provides the _() and N_() macros used to mark strings for translation. */
+#include <glib/gi18n.h>
 #else
 /* Provide transparent definitions if NLS is disabled. */
 #define	_(str)	str
