@@ -12,9 +12,10 @@
 
 #include <stdarg.h>
 
-#include "keyboard.h"
 #include "cmdseq.h"
 
+#include "keyboard.h"
+
 /* ----------------------------------------------------------------------------------------- */
 
 typedef struct {
@@ -211,10 +212,10 @@ void kbd_context_entry_remove(KbdContext *ctx, const gchar *key)
 ** 2000-02-09 -	Now knows about, and applies, the modifier mask allowing e.g. NumLock to be
 **		ignored.
 */
-static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
+static gboolean evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	KbdContext	*ctx = user;
-	KbdEntry	*ent;
+	KbdEntry	*ent = NULL;
 	gchar		*key;
 	gint		ret;
 	GdkModifierType	mod;
@@ -238,17 +239,17 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 					ent->entry.gdkevent.func((GdkEvent *) evt, ent->entry.gdkevent.user);
 					break;
 				case KET_GTKOBJECT:
-					gtk_signal_emit_by_name(GTK_OBJECT(ent->entry.object.obj),
+					g_signal_emit_by_name(G_OBJECT(ent->entry.object.obj),
 								ent->entry.object.signame, ent->entry.object.user, &ret);
 					break;
 				case KET_CMDSEQ:
 					csq_execute(ctx->min, ent->entry.cmdseq.cmdseq);
 					break;
 			}
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(wid), "key_press_event");
 		}
 		g_free(key);
 	}
+	return ent != NULL;
 }
 
 /* 1999-03-10 -	Attach a keyboard context to a window. This will cause all keyboard events
