@@ -106,7 +106,7 @@ Menu * mnu_menu_new(MenuInfo *mi, const gchar *name)
 		return NULL;
 
 	m = g_malloc(sizeof *m);
-	stu_strncpy(m->name, name, sizeof m->name);
+	g_strlcpy(m->name, name, sizeof m->name);
 	m->item = g_array_new(FALSE, FALSE, sizeof (MenuItem));
 	m->used = FALSE;
 
@@ -148,7 +148,7 @@ void mnu_menu_append_simple(Menu *m, const gchar *label, const gchar *cmdseq)
 	MenuItem	mi;
 
 	mi.type = ITEM_SIMPLE;
-	stu_strncpy(mi.data.simple.label.label, label, sizeof mi.data.simple.label.label);
+	g_strlcpy(mi.data.simple.label.label, label, sizeof mi.data.simple.label.label);
 	mi.data.simple.label.icon[0] = '\0';
 	mi.data.simple.cmdseq = g_string_new(cmdseq);
 	menu_append(m, &mi);
@@ -167,9 +167,9 @@ void mnu_menu_append_submenu(Menu *m, const gchar *label, const gchar *menu)
 	MenuItem	mi;
 
 	mi.type = ITEM_SUBMENU;
-	stu_strncpy(mi.data.submenu.label.label, label, sizeof mi.data.submenu.label.label);
+	g_strlcpy(mi.data.submenu.label.label, label, sizeof mi.data.submenu.label.label);
 	mi.data.submenu.label.icon[0] = '\0';
-	stu_strncpy(mi.data.submenu.menu, menu, sizeof mi.data.submenu.menu);
+	g_strlcpy(mi.data.submenu.menu, menu, sizeof mi.data.submenu.menu);
 	menu_append(m, &mi);
 }
 
@@ -214,7 +214,7 @@ static void evt_menu_action_activate(GtkWidget *wid, gpointer user)
 {
 	MainInfo	*min;
 
-	min = gtk_object_get_user_data(GTK_OBJECT(wid));
+	min = g_object_get_data(G_OBJECT(wid), "user");
 	csq_execute_format(min, "FileAction action='%s'", user);
 }
 
@@ -228,16 +228,22 @@ static void menu_build_action(MainInfo *min, GtkWidget *menu)
 
 	if((slist = dp_get_selection(dp)) != NULL)
 	{
-		GSList	*iter, *styles = NULL, *actions = NULL;
-		GList	*palist, *piter;
-		Style	*stl;
+		GSList		*iter, *styles = NULL, *actions = NULL;
+		GList		*palist, *piter;
+		GtkTreeModel	*tm = dp_get_tree_model(dp);
+		Style		*stl;
 
 		/* Build list of all styles used by rows in selection. */
 		for(iter = slist; iter != NULL; iter = g_slist_next(iter))
 		{
-			stl = DP_SEL_TYPE(iter)->style;
-			if(g_slist_find(styles, stl) == NULL)
-				styles = g_slist_prepend(styles, stl);
+			const FType	*ftype = dp_row_get_ftype(tm, iter->data);
+
+			if(ftype != NULL)
+			{
+				stl = ftype->style;
+				if(g_slist_find(styles, stl) == NULL)
+					styles = g_slist_prepend(styles, stl);
+			}
 		}
 
 		/* Compute intersection of all styles' action properties. */
@@ -262,9 +268,9 @@ static void menu_build_action(MainInfo *min, GtkWidget *menu)
 			for(iter = actions; iter != NULL; iter = g_slist_next(iter))
 			{
 				item = gtk_menu_item_new_with_label(iter->data);
-				gtk_object_set_user_data(GTK_OBJECT(item), min);
+				g_object_set_data(G_OBJECT(item), "user", min);
 				g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_action_activate), iter->data);
-				gtk_menu_append(GTK_MENU(menu), item);
+				gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 				gtk_widget_show(item);
 			}
 			g_slist_free(actions);
@@ -277,7 +283,7 @@ static void menu_build_action(MainInfo *min, GtkWidget *menu)
 
 		item = gtk_menu_item_new_with_label(_("(No Selection)"));
 		gtk_widget_set_sensitive(item, FALSE);
-		gtk_menu_append(GTK_MENU(menu), item);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		gtk_widget_show(item);
 	}
 }
@@ -299,21 +305,21 @@ static void menu_build_time(MainInfo *min, GtkWidget *menu)
 	strftime(buf, sizeof buf, "%H:%M.%S", nowf);
 
 	item = gtk_menu_item_new_with_label(buf);
-	gtk_menu_append(GTK_MENU(menu), item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	gtk_widget_show(item);
 }
 
 /* 2004-04-18 -	User picked an item from the <ParentsMenu> menu. Go there. */
 static void evt_menu_parents_activate(GtkWidget *wid, gpointer user)
 {
-	gchar	*text = NULL;
+	const gchar	*text = NULL;
 
 	if(!GTK_IS_MENU_ITEM(wid))
 		return;
 	wid = GTK_BIN(wid)->child;
 	if(!GTK_IS_LABEL(wid))
 		return;
-	gtk_label_get(GTK_LABEL(wid), &text);
+	text = gtk_label_get_text(GTK_LABEL(wid));
 	if(text != NULL)
 		csq_execute_format(user, "DirEnter 'dir=%s'", stu_escape(text));
 }
@@ -335,12 +341,12 @@ static void menu_build_parents(MainInfo *min, GtkWidget *menu)
 			break;
 		item = gtk_menu_item_new_with_label(canon);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_parents_activate), min);
-		gtk_menu_append(GTK_MENU(menu), item);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	}
 	/* Always append a root ("/") item, since the root has itself as its parent. Avoids empty menu, as bonus. */
 	item = gtk_menu_item_new_with_label("/");
 	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_parents_activate), min);
-	gtk_menu_append(GTK_MENU(menu), item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	gtk_widget_show_all(menu);
 }
 
@@ -351,22 +357,23 @@ static void evt_menu_activate(GtkWidget *wid, gpointer user)
 {
 	MainInfo	*min;
 
-	min = gtk_object_get_user_data(GTK_OBJECT(wid));
+	min = g_object_get_data(G_OBJECT(wid), "user");
 	csq_execute(min, user);
 }
 
 static void evt_menu_show(GtkWidget *wid, gpointer user)
 {
 	DynMenu	*dm = user;
-	dm->build(gtk_object_get_user_data(GTK_OBJECT(wid)), wid);
+	dm->build(g_object_get_data(G_OBJECT(wid), "user"), wid);
 }
 
 static void evt_menu_hide(GtkWidget *wid, gpointer user)
 {
-	GList	*iter;
+	GList	*list, *iter;
 
-	for(iter = gtk_container_children(GTK_CONTAINER(wid)); iter != NULL; iter = g_list_next(iter))
+	for(iter = list = gtk_container_get_children(GTK_CONTAINER(wid)); iter != NULL; iter = g_list_next(iter))
 		gtk_container_remove(GTK_CONTAINER(wid), iter->data);
+	g_list_free(list);
 }
 
 static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, Menu *parent)
@@ -387,7 +394,7 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 	if((d = g_hash_table_lookup(dynamic_hash, name)) != NULL)
 	{
 		menu = gtk_menu_new();
-		gtk_object_set_user_data(GTK_OBJECT(menu), mi->min);
+		g_object_set_data(G_OBJECT(menu), "user", mi->min);
 		g_signal_connect(G_OBJECT(menu), "show", G_CALLBACK(evt_menu_show), d);
 		g_signal_connect(G_OBJECT(menu), "hide", G_CALLBACK(evt_menu_hide), d);
 		return menu;
@@ -406,7 +413,7 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 				menuitem = gtk_menu_item_new_with_label(mit->data.simple.label.label);
 				g_signal_connect(G_OBJECT(menuitem), "activate",
 						 G_CALLBACK(evt_menu_activate), mit->data.simple.cmdseq->str);
-				gtk_object_set_user_data(GTK_OBJECT(menuitem), mi->min);
+				g_object_set_data(G_OBJECT(menuitem), "user", mi->min);
 				break;
 			case ITEM_SEPARATOR:
 				menuitem = gtk_menu_item_new();
@@ -425,7 +432,7 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 		}
 		if(menuitem)
 		{
-			gtk_menu_append(GTK_MENU(menu), menuitem);
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 			gtk_widget_show(menuitem);
 		}
 	}
@@ -469,7 +476,7 @@ static void evt_popup_hide(GtkWidget *wid, gpointer user)
 	gboolean	ret;
 
 	gui_events_flush();
-	gtk_signal_emit_by_name(GTK_OBJECT(wid), "destroy_event", NULL, &ret);
+	g_signal_emit_by_name(G_OBJECT(wid), "destroy_event", NULL, &ret);
 }
 
 gboolean mnu_menu_popup(const MenuInfo *mi, const gchar *name, GtkMenuPositionFunc pos, gpointer data, guint button, guint32 activate_time)
@@ -557,6 +564,7 @@ MenuInfo * mnu_menuinfo_copy(const MenuInfo *mi)
 	return nmi;
 }
 
+#if 0
 static void append_menu(gpointer key, gpointer value, gpointer user)
 {
 	GList	**list = user;
@@ -615,6 +623,7 @@ const gchar * mnu_menuinfo_select(const MenuInfo *mi)
 
 	return choice == 0 ? (selected ? selected : MENU_NONE) : NULL;
 }
+#endif
 
 static void destroy_menu(gpointer key, gpointer value, gpointer user)
 {
