@@ -27,7 +27,7 @@
 static struct {
 	guint		level;		/* Current begin/end nesting level. Keep first!! */
 	guint32		flags;		/* Flags from original progress_begin() call. */
-	gboolean	cancel;		/* Gets set as soon as user clicks the button. */
+	GCancellable	*cancel;	/* This is from the future. */
 	gboolean	show_byte;	/* Show total bytes progress bar? */
 	gboolean	show_item;	/* Show the item bar? */
 	gboolean	delayed;	/* Set to 1 during initial display delay. */
@@ -73,7 +73,7 @@ static guint count_toplevel_selected(MainInfo *min, guint64 *size)
 	for(iter = slist = dp_get_selection(dp); iter != NULL; iter = g_slist_next(iter), num++)
 	{
 		if(size)
-			*size += (guint64) DP_SEL_LSTAT(iter).st_size;
+			*size += dp_row_get_size(dp_get_tree_model(dp), iter->data);
 	}
 	dp_free_selection(slist);
 	return num;
@@ -94,41 +94,40 @@ static guint count_recursive_selected(MainInfo *min, guint64 *size)
 	for(iter = slist = dp_get_selection(dp); iter != NULL; iter = g_slist_next(iter), num++)
 	{
 		if(size)
-			*size += (gsize) DP_SEL_LSTAT(iter).st_size;
-		if(S_ISDIR(DP_SEL_LSTAT(iter).st_mode))
+			*size += dp_row_get_size(dp_get_tree_model(dp), iter->data);
+		if(dp_row_get_file_type(dp_get_tree_model(dp), iter->data, TRUE) == G_FILE_TYPE_DIRECTORY)
 		{
 			FUCount	fu;
+			GFile	*dir;
 
-			if(fut_dir_size(min, dp_full_name(dp, DP_SEL_INDEX(dp, iter)), size, &fu))
+			dir = dp_get_file_from_row(dp, iter->data);
+			if(fut_size_gfile(min, dir, size, &fu, NULL))
 				num += fu.num_total;
+			g_object_unref(dir);
 		}
 	}
 	dp_free_selection(slist);
+
 	return num;
 }
 
 static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 {
-	*(gboolean *) user = TRUE;
-	errno = EINTR;
+	g_cancellable_cancel(user);
 	return TRUE;
 }
 
 /* 1998-09-30 -	A callback for the big "Cancel" button. */
 static gint evt_cancel_clicked(GtkWidget *wid, gpointer user)
 {
-	*(gboolean *) user = TRUE;
-	errno = EINTR;
+	g_cancellable_cancel(user);
 	return TRUE;
 }
 
 static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	if(evt->keyval == GDK_Escape)
-	{
-		*(gboolean *) user = TRUE;
-		errno = EINTR;
-	}
+		g_cancellable_cancel(user);
 	return TRUE;
 }
 
@@ -146,7 +145,6 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 			flags &= PFLG_BUSY_MODE;	/* Clear any other flag. */
 
 		pgs_info.flags	    = flags;
-		pgs_info.cancel	    = FALSE;
 		pgs_info.show_byte  = (flags & PFLG_BYTE_VISIBLE) ? TRUE : FALSE;
 		pgs_info.show_item  = (flags & PFLG_ITEM_VISIBLE) ? TRUE : FALSE;
 		pgs_info.delayed    = TRUE;
@@ -154,6 +152,11 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		pgs_info.byte_pos   = 0;
 		pgs_info.last_secs  = 0U;
 
+		if(pgs_info.cancel == NULL)
+			pgs_info.cancel = g_cancellable_new();
+		else
+			g_cancellable_reset(pgs_info.cancel);
+
 		if(flags & PFLG_COUNT_RECURSIVE)
 			pgs_info.tot_num = count_recursive_selected(min, &pgs_info.byte_tot);
 		else
@@ -164,17 +167,15 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		pgs_info.dlg  = gtk_dialog_new();
 		gtk_widget_set_size_request(pgs_info.dlg, 384, -1);
 		pgs_info.body = gtk_label_new(op_name);
-		g_signal_connect(G_OBJECT(pgs_info.dlg), "delete_event", G_CALLBACK(evt_delete), &pgs_info.cancel);
-		g_signal_connect(G_OBJECT(pgs_info.dlg), "key_press_event", G_CALLBACK(evt_keypress), &pgs_info.cancel);
+		g_signal_connect(G_OBJECT(pgs_info.dlg), "delete_event", G_CALLBACK(evt_delete), pgs_info.cancel);
+		g_signal_connect(G_OBJECT(pgs_info.dlg), "key_press_event", G_CALLBACK(evt_keypress), pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.body, FALSE, FALSE, 0);
 
 		hbox = gtk_hbox_new(FALSE, 0);
 		pgs_info.tot_pgs = gtk_progress_bar_new();
 		if(flags & PFLG_BUSY_MODE)
 		{
-			gtk_progress_set_activity_mode(GTK_PROGRESS(pgs_info.tot_pgs), TRUE);
-			gtk_progress_bar_set_activity_step(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 3);
-			gtk_progress_bar_set_activity_blocks(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 10);
+			gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 1.f / 10.f);
 			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.tot_pgs, TRUE, TRUE, 0);
 		}
 		else
@@ -186,12 +187,12 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 			label = gtk_label_new(buf);
 			gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 		}
-		gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 0.0f);
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 0.f);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
 
 		if(pgs_info.show_byte)
 		{
-			sze_put_size64(size_buf, sizeof size_buf, pgs_info.byte_tot, SZE_AUTO);
+			sze_put_offset(size_buf, sizeof size_buf, pgs_info.byte_tot, SZE_AUTO, 3, ',');
 			g_snprintf(buf, sizeof buf, _("Total (%s)"), size_buf);
 			label = gtk_label_new(buf);
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), label, FALSE, FALSE, 0);
@@ -206,7 +207,7 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 			hbox = gtk_hbox_new(FALSE, 0);
 			pgs_info.item_pgs = gtk_progress_bar_new();
 			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.item_pgs, TRUE, TRUE, 0);
-			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.item_pgs), 0.0f);
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.item_pgs), 0.f);
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
 		}
 		if(!(flags & PFLG_BUSY_MODE))
@@ -217,7 +218,7 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		else
 			pgs_info.eta = NULL;
 		cancel = gtk_button_new_with_label(_("Cancel"));
-		g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(evt_cancel_clicked), &pgs_info.cancel);
+		g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(evt_cancel_clicked), pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->action_area), cancel, TRUE, TRUE, 0);
 
 		g_get_current_time(&pgs_info.time_begin);
@@ -227,6 +228,11 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 	pgs_info.level++;
 }
 
+GCancellable * pgs_progress_get_cancellable(void)
+{
+	return pgs_info.cancel;
+}
+
 /* 1998-09-30 -	End a progress-reporting session. Closes down the dialog box. */
 void pgs_progress_end(MainInfo *min)
 {
@@ -244,21 +250,7 @@ void pgs_progress_end(MainInfo *min)
 void pgs_progress_item_begin(MainInfo *min, const gchar *name, off_t size)
 {
 	if(pgs_info.flags & PFLG_BUSY_MODE)
-	{
-		GtkAdjustment	*adj;
-		gfloat		val;
-		static gint	scale = 10;
-
-		if(!--scale)
-		{
-			adj = GTK_PROGRESS(pgs_info.tot_pgs)->adjustment;
-			val = adj->value + 1;
-			if(val > adj->upper)
-				val = adj->lower;
-			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), val);
-			scale = 10;
-		}
-	}
+		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pgs_info.tot_pgs));
 	else
 		pgs_info.tot_pos++;
 
@@ -268,7 +260,7 @@ void pgs_progress_item_begin(MainInfo *min, const gchar *name, off_t size)
 	{
 		gchar	size_buf[32], buf[FILENAME_MAX + 32];
 
-		sze_put_size64(size_buf, sizeof size_buf, size, SZE_AUTO);
+		sze_put_offset(size_buf, sizeof size_buf, size, SZE_AUTO, 3, ',');
 		g_snprintf(buf, sizeof buf, "%s (%s)", name, size_buf);
 		gtk_label_set_text(GTK_LABEL(pgs_info.item), buf);
 	}
@@ -284,8 +276,6 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 
 	if(pgs_info.dlg != NULL)
 	{
-		gint	old_errno = errno;
-
 		g_get_current_time(&time_now);
 		if(pgs_info.delayed)
 		{
@@ -311,7 +301,7 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 					pgs_info.last_secs = secs;
 					spd = (pgs_info.byte_pos + pos) / secs;
 					eta = (pgs_info.byte_tot - (pgs_info.byte_pos + pos)) / spd;
-					sze_put_size(spdbuf, sizeof spdbuf, spd, SZE_AUTO);
+					sze_put_offset(spdbuf, sizeof spdbuf, spd, SZE_AUTO, 3, ',');
 					g_snprintf(buf, sizeof buf, _("Elapsed %02d:%02d  Speed %s/s  ETA %02u:%02u"),
 						pgs_info.last_secs / 60, pgs_info.last_secs % 60, spdbuf,
 						eta / 60, eta % 60);
@@ -323,12 +313,11 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 			gtk_widget_show_all(pgs_info.dlg);
 		}
 		if(pgs_info.show_byte && pgs_info.byte_tot)
-			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.byte_pgs), (gfloat) (pgs_info.byte_pos + pos) / pgs_info.byte_tot);
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.byte_pgs), (gfloat) (pgs_info.byte_pos + pos) / pgs_info.byte_tot);
 		if(pgs_info.show_item && pgs_info.item_size)
-			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.item_pgs), (gfloat) pos / pgs_info.item_size);
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.item_pgs), (gfloat) pos / pgs_info.item_size);
 		gui_events_flush();
-		errno = old_errno;
-		if(pgs_info.cancel)
+		if(g_cancellable_is_cancelled(pgs_info.cancel))
 			return PGS_CANCEL;
 	}
 	return PGS_PROCEED;
@@ -337,11 +326,8 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 /* 1998-09-30 -	Done with an item. */
 void pgs_progress_item_end(MainInfo *min)
 {
-	gint	old_errno = errno;
-
 	pgs_info.byte_pos += pgs_info.item_size;
-	gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), (gfloat) pgs_info.tot_pos / pgs_info.tot_num);
-	gui_events_flush();
 
-	errno = old_errno;
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.tot_pgs), (gfloat) pgs_info.tot_pos / pgs_info.tot_num);
+	gui_events_flush();
 }
