@@ -7,10 +7,12 @@
 
 typedef enum { PGS_CANCEL, PGS_PROCEED } PgsRes;
 
-#define	PFLG_COUNT_RECURSIVE	(1<<0)	/* Count # of items recursively? */
-#define	PFLG_ITEM_VISIBLE	(1<<1)	/* Display progress for individual items? */
-#define	PFLG_BYTE_VISIBLE	(1<<2)	/* Display total byte progress? */
-#define	PFLG_BUSY_MODE		(1<<10)	/* Just indicate "busyness", no direct filesystem coupling. */
+enum {
+	PFLG_COUNT_RECURSIVE	= (1<<0),	/* Count # of items recursively? */
+	PFLG_ITEM_VISIBLE	= (1<<1),	/* Display progress for individual items? */
+	PFLG_BYTE_VISIBLE	= (1<<2),	/* Display total byte progress? */
+	PFLG_BUSY_MODE		= (1<<10)	/* Just indicate "busyness", no direct filesystem coupling. */
+};
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -19,6 +21,8 @@ typedef enum { PGS_CANCEL, PGS_PROCEED } PgsRes;
 */
 extern void	pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags);
 
+extern GCancellable *	pgs_progress_get_cancellable(void);
+
 /* Use these three calls to report the progress of each item. */
 extern void	pgs_progress_item_begin(MainInfo *min, const gchar *name, off_t size);
 extern PgsRes	pgs_progress_item_update(MainInfo *min, off_t pos);
