@@ -24,7 +24,7 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include "convutil.h"
+#include "convstream.h"
 #include "errors.h"
 #include "fileutil.h"
 #include "dirpane.h"
@@ -33,7 +33,7 @@
 #include "strutil.h"
 #include "textview.h"
 
-#define	HEX_ROW_SIZE	(16)		/* Must be multiple of four! */
+#define	HEX_ROW_SIZE	(32)		/* Must be multiple of four! */
 #define	HEX_UNPRINTABLE	('.')
 
 /* ----------------------------------------------------------------------------------------- */
@@ -68,7 +68,9 @@ typedef struct {
 	GtkWidget	*blabel;		/* A label that is in the action box. */
 	GtkWidget	*bplabel;		/* This label shows file position. */
 	guint		search_pos;		/* Offset into text to start next search at. */
-	guint		keypress_handler;	/* Handler ID for the keypress event. */
+	gulong		delete_handler;		/* Handler ID for the delete event. */
+	gulong		keypress_handler;	/* Handler ID for the keypress event. */
+	ConvStream	*convstream;		/* Used to abstract away encoding issues. */
 } TxvInfo;
 
 /* ----------------------------------------------------------------------------------------- */
@@ -96,7 +98,7 @@ static void really_destroy(GtkWidget *wid)
 {
 	TxvInfo	*txi;
 
-	if((wid != NULL) && ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL))
+	if((wid != NULL) && ((txi = g_object_get_data(G_OBJECT(wid), "user")) != NULL))
 	{
 		if(goto_dialog.dlg != NULL)		/* Goto dialog window open? Then close it. */
 		{
@@ -119,42 +121,16 @@ static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 {
 	really_destroy(wid);
 
-	return TRUE;
+	return FALSE;
 }
 
-/* 2008-03-15 -	Helper routine since appending text now needs a bit more thinking to handle UTF-8.
- *		Returns FALSE is conversion failed and the caller should probably give up.
-*/
-static gboolean do_text_append(TxvInfo *txi, const gchar *text, gsize length)
+/* 2010-07-19 -	Rewritten in a much simpler way. Now, input is expected to be UTF-8, so we don't need to check/convert. */
+static void do_text_append(TxvInfo *txi, const gchar *text, gsize length)
 {
 	GtkTextIter	iter;
-	const gchar	*eptr;
 
 	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
-	if(g_utf8_validate(text, length, &eptr))
-		gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, text, length);
-	else
-	{
-		gchar	*utf8;
-		gsize	bin, bout;
-		GError	*err = NULL;
-
-		if((utf8 = g_locale_to_utf8(text, length, &bin, &bout, &err)) != NULL)
-		{
-			gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, utf8, bout);
-			g_free(utf8);
-		}
-		if(err != NULL)
-		{
-			const gchar	*local = NULL;
-
-			g_get_charset(&local);
-			err_printf(txi->min, _("Unable to convert text from native character set (%s) to UTF-8: %s"), local, err->message);
-			g_error_free(err);
-			return FALSE;
-		}
-	}
-	return TRUE;
+	gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, text, length);
 }
 
 /* 1999-03-02 -	Users of this module no longer have access to the GtkText widget, so
@@ -164,7 +140,7 @@ void txv_text_append(GtkWidget *wid, const gchar *text, gsize length)
 {
 	TxvInfo		*txi;
 
-	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL) || length < 1)
+	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL) || length < 1)
 		return;
 	do_text_append(txi, text, length);
 }
@@ -184,7 +160,7 @@ void txv_text_append_with_color(GtkWidget *wid, const gchar *text, gsize length,
 		txv_text_append(wid, text, length);
 		return;
 	}
-	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL) || length < 1)
+	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL) || length < 1)
 		return;
 	pos = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(txi->buffer));
 	do_text_append(txi, text, length);
@@ -200,6 +176,28 @@ void txv_text_append_with_color(GtkWidget *wid, const gchar *text, gsize length,
 	gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(txi->buffer), tag, &start, &end);
 }
 
+static void sink_text(const gchar *text, gsize length, gpointer user)
+{
+	TxvInfo	*txi = user;
+
+	/* Is the conversion stream trying to signal us an error? */
+	if(length == 0)
+	{
+		gchar	msg[256];
+		gsize	len;
+		GdkColor red;
+
+		len = g_snprintf(msg, sizeof msg, _("\n** Text conversion from charset '%s' failed, aborting."), text);
+		red.red = 0xffff;
+		red.green = 0;
+		red.blue = 0;
+		red.pixel = 0;
+		txv_text_append_with_color(txi->win, msg, len, &red);
+		return;
+	}
+	do_text_append(txi, text, length);
+}
+
 /* 1998-12-18 -	Rewrote this a couple of times today. Note how this version really doesn't
 **		care about the size of its input file; it just reads until reading fails.
 **		This makes it possible to look at /proc files, which have size 0.
@@ -211,47 +209,36 @@ void txv_text_append_with_color(GtkWidget *wid, const gchar *text, gsize length,
 **		using normal file I/O if mmap() fails, which can happen on certain filesystems.
 ** 1999-03-02 -	New interface, renamed.
 ** 1999-04-06 -	Added the <buf_size> argument, which is used when mmap() isn't.
+** 2010-02-28 -	Ported to GIO, mmap support removed (sadly).
 */
-gint txv_text_load(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size)
+gint txv_text_load(GtkWidget *wid, GFile *file, gsize buf_size, const gchar *encoding, GError **err)
 {
-	gchar	*buf;
-	gint	fd, ret = 0;
-	TxvInfo	*txi;
+	GFileInputStream	*is;
+	TxvInfo			*txi;
 
-	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL) || (name == NULL))
+	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL) || (file == NULL))
 		return 0;
 
-	txv_set_label_from_filename(wid, name);
+	txv_set_label_from_file(wid, file);
 
 	err_clear(txi->min);
-	if((fd = open(name, O_RDONLY)) >= 0)
+	if((is = g_file_read(file, NULL, err)) != NULL)
 	{
-		gsize	size;
+		gchar		*buf;
+		gssize		got;
+		ConvStream	*cs;
 
-		if(use_mmap && fut_size(name, &size) && (size > 0) &&
-		   (buf = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0)) != MAP_FAILED)
+		buf = g_malloc(buf_size);
+		cs = convstream_new(encoding, 32 << 10, sink_text, txi);
+		while((got = g_input_stream_read(G_INPUT_STREAM(is), buf, buf_size, NULL, err)) > 0)
 		{
-			do_text_append(txi, buf, size);
-			munmap(buf, size);
+			convstream_source(cs, buf, got);
 		}
-		else
-		{
-			gssize	got;
-
-			buf = g_malloc(buf_size);
-			while((got = read(fd, buf, buf_size)) > 0)
-			{
-				if(!do_text_append(txi, buf, got))
-					break;
-			}
-			g_free(buf);
-		}
-		close(fd);
+		convstream_destroy(cs);
+		g_free(buf);
+		g_object_unref(is);
 	}
-	if(errno != 0)
-		err_set(txi->min, errno, _("File reading"), (gchar *) name);
-
-	return ret;
+	return *err == NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -292,21 +279,20 @@ static void build_hex_text(const guchar *data, gchar *text, gsize bytes)
 **		number of bytes at a time, it was grossly inefficient to load the entire file.
 **		Now we do plenty of small read()s instead, which I'm sure is slower, but at
 **		least doesn't waste memory.
+** 2010-02-28 -	Ported to use GIO.
 */
-gint txv_text_load_hex(GtkWidget *win, const gchar *name)
+gint txv_text_load_hex(GtkWidget *win, GFile *file, GError **err)
 {
 	gchar		line[8 + 1 + (HEX_ROW_SIZE / 4) * 9 + 1 + HEX_ROW_SIZE + 1 + 1],
 			group[9 * (HEX_ROW_SIZE / 4) + 1], text[HEX_ROW_SIZE + 1];
-	guchar		buf[HEX_ROW_SIZE];
-	gint		fd, len, chunk;
-	gsize		offset = 0;
 	TxvInfo		*txi;
-/*	GtkTextTag	*tag;*/
+	GFileInputStream *is;
+	GtkTextTag	*tag;
 
-	if((win == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(win))) == NULL) || (name == NULL))
+	if((win == NULL) || ((txi = g_object_get_data(G_OBJECT(win), "user")) == NULL) || (file == NULL))
 		return 0;
 
-	txv_set_label_from_filename(win, name);
+	txv_set_label_from_file(win, file);
 
 	memset(group, ' ', sizeof group - 1);
 	group[sizeof group - 1] = '\0';
@@ -314,12 +300,14 @@ gint txv_text_load_hex(GtkWidget *win, const gchar *name)
 	text[sizeof text - 1]   = '\0';
 
 	err_clear(txi->min);
-	if((fd = open(name, O_RDONLY)) >= 0)
+	if((is = g_file_read(file, NULL, err)) != NULL)
 	{
+		guchar		buf[HEX_ROW_SIZE];
 		GtkTextIter	iter;
+		gsize		offset = 0, chunk, len;
 
 		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
-		while((chunk = read(fd, buf, sizeof buf)) > 0)
+		while((chunk = g_input_stream_read(G_INPUT_STREAM(is), buf, sizeof buf, NULL, err)) > 0)
 		{
 			if(chunk < sizeof buf)
 			{
@@ -332,15 +320,15 @@ gint txv_text_load_hex(GtkWidget *win, const gchar *name)
 			gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, line, len);
 			offset += chunk;
 		}
-		close(fd);
+		g_object_unref(is);
 	}
 
-/*	if((tag = gtk_text_tag_new("fixed")) != NULL)
+	if((tag = gtk_text_tag_new("fixed")) != NULL)
 	{
 		GtkTextIter	start, end;
 		GtkTextTagTable	*tt;
 
-		g_object_set(G_OBJECT(tag), "font", "fixed", NULL);
+		g_object_set(G_OBJECT(tag), "family", "Monospace", NULL);
 		tt = gtk_text_buffer_get_tag_table(GTK_TEXT_BUFFER(txi->buffer));
 		gtk_text_tag_table_add(tt, tag);
 		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &start);
@@ -348,10 +336,8 @@ gint txv_text_load_hex(GtkWidget *win, const gchar *name)
 		gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(txi->buffer), tag, &start, &end);
 		g_object_unref(G_OBJECT(tag));
 	}
-*/
-	if(errno != 0)
-		err_set(txi->min, errno, _("File reading"), (gchar *) name);
-	return errno != 0;
+
+	return *err == NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -376,7 +362,7 @@ static void reset_search(TxvInfo *txi)
 */
 static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
-	TxvInfo	*txi = gtk_object_get_user_data(GTK_OBJECT(wid));
+	TxvInfo	*txi = g_object_get_data(G_OBJECT(wid), "user");
 
 	if(evt->keyval == GDK_Escape || evt->keyval == GDK_Left)
 		really_destroy(wid);
@@ -428,36 +414,33 @@ static gint evt_pos_changed(GtkAdjustment *adj, gpointer user)
 static gint evt_move_clicked(GtkWidget *wid, gpointer user)
 {
 	TxvInfo		*txi = user;
-	GtkAdjustment	*adj;
 	GtkTextIter	iter;
 	gint		bx, by;
 
 	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(txi->text), GTK_TEXT_WINDOW_WIDGET, 0, 0, &bx, &by);
 	gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(txi->text), &iter, by, NULL);
 
-	if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
+	switch((MoveID) GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wid), "user")))
 	{
-		switch((MoveID) GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid))))
-		{
-			case MVE_UP:
-				gtk_text_view_backward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
-				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
-				break;
-			case MVE_DOWN:
-				gtk_text_view_forward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
-				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
-				break;
-			case MVE_TOP:
-				gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
-				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
-				reset_search(txi);
-				break;
-			case MVE_BOTTOM:
-				gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
-				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
-				reset_search(txi);
-				break;
-		}
+		case MVE_UP:
+			gtk_text_view_backward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
+			gtk_text_view_backward_display_line_start(GTK_TEXT_VIEW(txi->text), &iter);
+			gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
+			break;
+		case MVE_DOWN:
+			gtk_text_view_forward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
+			gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
+			break;
+		case MVE_TOP:
+			gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+			gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
+			reset_search(txi);
+			break;
+		case MVE_BOTTOM:
+			gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+			gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 1.0);
+			reset_search(txi);
+			break;
 	}
 	return TRUE;
 }
@@ -482,7 +465,7 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 	{
 		const gchar	*txt;
 		gchar		unit;
-		gint		value;
+		gint		value, line;
 		GtkTextIter	iter;
 
 		txt = gtk_entry_get_text(GTK_ENTRY(goto_dialog.entry));
@@ -494,8 +477,8 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 			switch(unit)
 			{
 			case '%':
-				value = (adj->upper * value) / 100;
-				gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(txi->buffer), &iter, value);
+				line = (gtk_text_buffer_get_line_count(GTK_TEXT_BUFFER(txi->buffer)) * value) / 100;
+				gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(txi->buffer), &iter, line);
 				break;
 			case 'M':
 			case 'm':
@@ -529,7 +512,7 @@ static gint evt_close_clicked(GtkWidget *wid, gpointer user)
 		win = txi->win;
 		really_destroy(win);
 	}
-	return TRUE;
+	return FALSE;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -612,7 +595,8 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 	srch_dialog.vbox  = gtk_vbox_new(FALSE, 0);
 	srch_dialog.label = gtk_label_new(_("Enter Text to Search For (RE)"));
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.label, FALSE, FALSE, 0);
-	srch_dialog.entry = gtk_entry_new_with_max_length(SEARCH_RE_SIZE - 1);
+	srch_dialog.entry = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(srch_dialog.entry), SEARCH_RE_SIZE - 1);
 	gtk_entry_set_text(GTK_ENTRY(srch_dialog.entry), last_search.re);
 	gtk_editable_select_region(GTK_EDITABLE(srch_dialog.entry), 0, -1);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.entry, FALSE, FALSE, 0);
@@ -634,7 +618,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 		GString	*tmp;
 
 		txt = gtk_entry_get_text(GTK_ENTRY(srch_dialog.entry));
-		stu_strncpy(last_search.re, txt, sizeof last_search.re);
+		g_strlcpy(last_search.re, txt, sizeof last_search.re);
 		last_search.no_case = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase));
 		last_search.no_lf   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnolf));
 		if(txt != NULL && (tmp = g_string_new("(")) != NULL)
@@ -695,11 +679,12 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	txi->min = min;
 	txi->search_pos = 0;		/* Start searching from the beginning. */
-	txi->keypress_handler = 0;
+	txi->delete_handler = 0u;
+	txi->keypress_handler = 0u;
 
 	txi->win = win_window_open(min->cfg.wininfo, WIN_TEXTVIEW);
-	gtk_object_set_user_data(GTK_OBJECT(txi->win), txi);
-	g_signal_connect(G_OBJECT(txi->win), "delete_event", G_CALLBACK(evt_delete), txi);
+	g_object_set_data(G_OBJECT(txi->win), "user", txi);
+	txv_connect_delete(txi->win, NULL, NULL);
 	txv_connect_keypress(txi->win, NULL, NULL);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -711,8 +696,8 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	txi->scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(txi->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	g_signal_connect_after(GTK_OBJECT(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))),
-				"value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), txi);
+	g_signal_connect_after(G_OBJECT(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))),
+				"value_changed", G_CALLBACK(evt_pos_changed), txi);
 	gtk_container_add(GTK_CONTAINER(txi->scwin), txi->text);
 
 	gtk_box_pack_start(GTK_BOX(vbox), txi->scwin, TRUE, TRUE, 0);
@@ -732,37 +717,37 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	accel = gtk_accel_group_new();
 
-	btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
-	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_UP));
+	btn = gui_stock_button_new(GTK_STOCK_GO_UP);
+	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_UP));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
-	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_DOWN));
+	btn = gui_stock_button_new(GTK_STOCK_GO_DOWN);
+	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_DOWN));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
-	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_TOP));
+	btn = gui_stock_button_new(GTK_STOCK_GOTO_TOP);
+	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_TOP));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_Home, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_BOTTOM);
-	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_BOTTOM));
+	btn = gui_stock_button_new(GTK_STOCK_GOTO_BOTTOM);
+	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_BOTTOM));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_End, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 
 	bhbox = gtk_hbox_new(FALSE, 0);
-	btn = gtk_button_new_from_stock(GTK_STOCK_JUMP_TO);
+	btn = gui_stock_button_new(GTK_STOCK_JUMP_TO);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_goto_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 
-	btn = gtk_button_new_from_stock(GTK_STOCK_FIND);
+	btn = gui_stock_button_new(GTK_STOCK_FIND);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_F, GDK_CONTROL_MASK, 0);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_search_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), bhbox, FALSE, FALSE, 5);
 
-	btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	btn = gui_stock_button_new(GTK_STOCK_CLOSE);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_close_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, TRUE, 0);
 	gtk_widget_set_sensitive(txi->bhbox, FALSE);
@@ -787,27 +772,47 @@ void txv_show(GtkWidget *textviewer)
 	win_window_show(textviewer);
 }
 
+/* 2010-03-13 -	Factored this one out of txv_connect_delete() and txv_connect_keypress, below. */
+static gulong connect_handler(TxvInfo *txi, gulong *handler, const gchar *signame, GCallback func, GCallback default_func, gpointer user)
+{
+	if(*handler != 0u)
+	{
+		g_signal_handler_disconnect(G_OBJECT(txi->win), *handler);
+		*handler = 0;
+	}
+	if(func == NULL)		/* Reset internal handler? */
+	{
+		func = default_func;
+		user = txi->win;
+	}
+	*handler = g_signal_connect(G_OBJECT(txi->win), signame, func, user);
+	return *handler;
+}
+
+/* 2010-03-13 -	Replace the module's own default delete handler with a custom handler.
+**		Note that in order to not leak memory, any custom handler must call
+**		txv_close() itself to clean up after the view.
+*/
+gulong txv_connect_delete(GtkWidget *wid, GCallback func, gpointer user)
+{
+	TxvInfo	*txi;
+
+	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL))
+		return 0;
+	return connect_handler(txi, &txi->delete_handler, "delete_event", func, G_CALLBACK(evt_delete), user);
+}
+
 /* 1999-03-02 -	Install a keypress handler on the textviewing window. This cannot be
 **		done directly by the module user, since we have an internal handler
 **		and they don't seem to chain properly... Odd.
 */
-guint txv_connect_keypress(GtkWidget *wid, GtkSignalFunc func, gpointer user)
+gulong txv_connect_keypress(GtkWidget *wid, GCallback func, gpointer user)
 {
 	TxvInfo	*txi;
 
-	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL))
+	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL))
 		return 0;
-	if(txi->keypress_handler > 0)
-	{
-		gtk_signal_disconnect(GTK_OBJECT(wid), txi->keypress_handler);
-		txi->keypress_handler = 0;
-	}
-	if(func == NULL)		/* Reset internal handler? */
-	{
-		func = GTK_SIGNAL_FUNC(evt_keypress);
-		user = wid;
-	}
-	return txi->keypress_handler = g_signal_connect(G_OBJECT(wid), "key_press_event", G_CALLBACK(func), user);
+	return connect_handler(txi, &txi->keypress_handler, "key_press_event", func, G_CALLBACK(evt_keypress), user);
 }
 
 /* 1999-01-05 -	Set the label. This is kind of redundant, but true to the original Opus look. :)
@@ -817,22 +822,25 @@ void txv_set_label(GtkWidget *wid, const gchar *text)
 {
 	TxvInfo	*txi;
 
-	if(wid != NULL && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL && text != NULL)
+	if(wid != NULL && (txi = g_object_get_data(G_OBJECT(wid), "user")) != NULL && text != NULL)
 	{
 		win_window_set_title(wid, text);
 		gtk_label_set_text(GTK_LABEL(txi->blabel), text);
 	}
 }
 
-/* 2009-05-27 -	Set the label of the view, based on a filename input. This needs encoding-conversion. */
-void txv_set_label_from_filename(GtkWidget *wid, const gchar *filename)
+/* 2009-05-27 -	Set the label of the view, based on a filename input. This needs encoding-conversion.
+** 2010-02-28 -	Converted to GIO.
+*/
+void txv_set_label_from_file(GtkWidget *wid, GFile *file)
 {
-	Conv	conv;
-	gchar	*dpl;
+	gchar	*dn;
 
-	dpl = conv_begin_reverse(&conv, filename);
-	txv_set_label(wid, dpl);
-	conv_end(&conv);
+	if((dn = g_file_get_parse_name(file)) != NULL)
+	{
+		txv_set_label(wid, dn);
+		g_free(dn);
+	}
 }
 
 /* 1999-03-02 -	Enable the text widget for viewing. Called once all insertions are done. */
@@ -840,7 +848,7 @@ void txv_enable(GtkWidget *wid)
 {
 	TxvInfo	*txi;
 
-	if((wid != NULL) && ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL))
+	if((wid != NULL) && ((txi = g_object_get_data(G_OBJECT(wid), "user")) != NULL))
 	{
 		gtk_widget_show(wid);	/* Just in case. */
 /*		gtk_adjustment_set_value(GTK_ADJUSTMENT(GTK_TEXT(txi->text)->vadj), 0.0f);*/
@@ -857,10 +865,6 @@ void txv_close(GtkWidget *wid)
 {
 	TxvInfo	*txi;
 
-	if(wid != NULL && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
-	{
-		GtkWidget	*win = txi->win;
-
-		really_destroy(win);		/* This makes the TxvInfo go away. */
-	}
+	if(wid != NULL && (txi = g_object_get_data(G_OBJECT(wid), "user")) != NULL)
+		really_destroy(txi->win);		/* This makes the TxvInfo go away. */
 }
